@@ -188,7 +188,11 @@ static GSList *__log_functions = NULL;
 
 static gint __default_level;
 static gint __use_color;
-gboolean __gst_debug_enabled = TRUE;
+
+/* disabled by default, as soon as some threshold is set > NONE,
+ * it becomes enabled. */
+gboolean __gst_debug_enabled = FALSE;
+static gboolean __gst_debug_min = GST_LEVEL_NONE;
 
 
 GstDebugCategory *GST_CAT_DEFAULT = NULL;
@@ -222,6 +226,7 @@ GstDebugCategory *GST_CAT_CALL_TRACE = NULL;
 GstDebugCategory *GST_CAT_SIGNAL = NULL;
 GstDebugCategory *GST_CAT_PROBE = NULL;
 GstDebugCategory *GST_CAT_REGISTRY = NULL;
+GstDebugCategory *GST_CAT_QOS = NULL;
 
 /* FIXME: export this? */
 gboolean
@@ -239,11 +244,9 @@ __gst_in_valgrind (void)
 #ifdef HAVE_VALGRIND
     if (RUNNING_ON_VALGRIND) {
       GST_CAT_INFO (GST_CAT_GST_INIT, "we're running inside valgrind");
-      VALGRIND_PRINTF
-          ("GStreamer has detected that it is running inside valgrind.");
-      VALGRIND_PRINTF
-          ("It might now take different code paths to ease debugging.");
-      VALGRIND_PRINTF ("Of course, this may also lead to different bugs.");
+      printf ("GStreamer has detected that it is running inside valgrind.\n");
+      printf ("It might now take different code paths to ease debugging.\n");
+      printf ("Of course, this may also lead to different bugs.\n");
       in_valgrind = GST_VG_INSIDE;
     } else {
       GST_CAT_LOG (GST_CAT_GST_INIT, "not doing extra valgrind stuff");
@@ -345,6 +348,7 @@ _gst_debug_init (void)
   GST_CAT_PROBE = _gst_debug_category_new ("GST_PROBE",
       GST_DEBUG_BOLD | GST_DEBUG_FG_GREEN, "pad probes");
   GST_CAT_REGISTRY = _gst_debug_category_new ("GST_REGISTRY", 0, "registry");
+  GST_CAT_QOS = _gst_debug_category_new ("GST_QOS", 0, "QoS");
 
 
   /* print out the valgrind message if we're in valgrind */
@@ -539,9 +543,9 @@ gst_debug_construct_term_color (guint colorinfo)
  * The default logging handler used by GStreamer. Logging functions get called
  * whenever a macro like GST_DEBUG or similar is used. This function outputs the
  * message and additional info using the glib error handler.
- * You can add other handlers by using #gst_debug_add_log_function.
+ * You can add other handlers by using gst_debug_add_log_function().
  * And you can remove this handler by calling
- * gst_debug_remove_log_function (gst_debug_log_default);
+ * gst_debug_remove_log_function(gst_debug_log_default);
  */
 void
 gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
@@ -789,6 +793,10 @@ void
 gst_debug_set_active (gboolean active)
 {
   __gst_debug_enabled = active;
+  if (active)
+    __gst_debug_min = GST_LEVEL_NONE;
+  else
+    __gst_debug_min = GST_LEVEL_COUNT;
 }
 
 /**
@@ -1004,6 +1012,9 @@ gst_debug_category_set_threshold (GstDebugCategory * category,
 {
   g_return_if_fail (category != NULL);
 
+  if (level > __gst_debug_min)
+    __gst_debug_enabled = TRUE;
+
   gst_atomic_int_set (&category->threshold, level);
 }
 
@@ -1086,8 +1097,6 @@ gst_debug_category_get_description (GstDebugCategory * category)
  * Returns a snapshot of a all categories that are currently in use . This list
  * may change anytime.
  * The caller has to free the list after use.
- * <emphasis>This function is not threadsafe, so only use it while only the
- * main thread is running.</emphasis>
  *
  * Returns: the list of categories
  */
