@@ -87,9 +87,10 @@
  * value from the READY to PAUSED state will be GST_STATE_CHANGE_NO_PREROLL.
  * </para>
  * <para>
- * A typical live source will timestamp the buffers they create with the 
- * current stream time of the pipeline. This is one reason that they can only 
- * produce data in PLAYING, when the clock is actually distributed and running.
+ * A typical live source will timestamp the buffers it creates with the 
+ * current stream time of the pipeline. This is one reason why a live source
+ * can only produce data in the PLAYING state, when the clock is actually 
+ * distributed and running.
  * </para>
  * <para>
  * The #GstBaseSrc::get_times method can be used to implement pseudo-live 
@@ -154,18 +155,23 @@
  * </programlisting>
  * </para>
  * <para>
- * Last reviewed on 2006-03-31 (0.10.5)
+ * Note that setting the source element to NULL or READY when the 
+ * pipeline is in the PAUSED state may cause a deadlock since the streaming
+ * thread might be blocked in PREROLL.
+ * </para>
+ * <para>
+ * Last reviewed on 2006-04-14 (0.10.6)
  * </para>
  * </refsect2>
  */
 
-#include <stdlib.h>
-#include <string.h>
-
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
+#include <stdlib.h>
+#include <string.h>
+
 #include "gstbasesrc.h"
 #include "gsttypefindhelper.h"
 #include <gst/gstmarshal.h>
@@ -212,6 +218,10 @@ struct _GstBaseSrcPrivate
   gboolean last_sent_eos;       /* last thing we did was send an EOS (we set this
                                  * to avoid the sending of two EOS in some cases) */
   gboolean discont;
+
+  /* two segments to be sent in the streaming thread with STREAM_LOCK */
+  GstEvent *close_segment;
+  GstEvent *start_segment;
 };
 
 static GstElementClass *parent_class = NULL;
@@ -292,8 +302,8 @@ gst_base_src_class_init (GstBaseSrcClass * klass)
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
 
-  gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
+  gobject_class = G_OBJECT_CLASS (klass);
+  gstelement_class = GST_ELEMENT_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (GstBaseSrcPrivate));
 
@@ -303,15 +313,15 @@ gst_base_src_class_init (GstBaseSrcClass * klass)
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_base_src_set_property);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_base_src_get_property);
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_BLOCKSIZE,
+  g_object_class_install_property (gobject_class, PROP_BLOCKSIZE,
       g_param_spec_ulong ("blocksize", "Block size",
           "Size in bytes to read per buffer", 1, G_MAXULONG, DEFAULT_BLOCKSIZE,
           G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_NUM_BUFFERS,
+  g_object_class_install_property (gobject_class, PROP_NUM_BUFFERS,
       g_param_spec_int ("num-buffers", "num-buffers",
           "Number of buffers to output before sending EOS", -1, G_MAXINT,
           DEFAULT_NUM_BUFFERS, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_TYPEFIND,
+  g_object_class_install_property (gobject_class, PROP_TYPEFIND,
       g_param_spec_boolean ("typefind", "Typefind",
           "Run typefind before negotiating", DEFAULT_TYPEFIND,
           G_PARAM_READWRITE));
@@ -405,8 +415,14 @@ gst_base_src_finalize (GObject * object)
  * @src: base source instance
  * @live: new live-mode
  *
- * If the element listens to a live source, the @livemode should
- * be set to %TRUE. This declares that this source can't seek.
+ * If the element listens to a live source, @live should
+ * be set to %TRUE. 
+ *
+ * A live source will not produce data in the PAUSED state and
+ * will therefore not be able to participate in the PREROLL phase
+ * of a pipeline. To signal this fact to the application and the 
+ * pipeline, the state change return value of the live source will
+ * be GST_STATE_CHANGE_NO_PREROLL.
  */
 void
 gst_base_src_set_live (GstBaseSrc * src, gboolean live)
@@ -797,12 +813,12 @@ gst_base_src_perform_seek (GstBaseSrc * src, GstEvent * event, gboolean unlock)
     gst_base_src_unlock (src);
 
   /* grab streaming lock, this should eventually be possible, either
-   * because the task is paused or out streaming thread stopped 
+   * because the task is paused or our streaming thread stopped 
    * because our peer is flushing. */
   GST_PAD_STREAM_LOCK (src->srcpad);
 
   /* make copy into temp structure, we can only update the main one
-   * when the subclass actually could to the seek. */
+   * when the subclass actually could do the seek. */
   memcpy (&seeksegment, &src->segment, sizeof (GstSegment));
 
   /* now configure the seek segment */
@@ -829,10 +845,12 @@ gst_base_src_perform_seek (GstBaseSrc * src, GstEvent * event, gboolean unlock)
     GST_DEBUG_OBJECT (src, "closing running segment %" G_GINT64_FORMAT
         " to %" G_GINT64_FORMAT, src->segment.start, src->segment.last_stop);
 
-    gst_pad_push_event (src->srcpad,
-        gst_event_new_new_segment (TRUE,
-            src->segment.rate, src->segment.format,
-            src->segment.start, src->segment.last_stop, src->segment.time));
+    if (src->priv->close_segment)
+      gst_event_unref (src->priv->close_segment);
+    src->priv->close_segment =
+        gst_event_new_new_segment_full (TRUE,
+        src->segment.rate, src->segment.applied_rate, src->segment.format,
+        src->segment.start, src->segment.last_stop, src->segment.time);
   }
 
   /* if successfull seek, we update our real segment and push
@@ -853,10 +871,12 @@ gst_base_src_perform_seek (GstBaseSrc * src, GstEvent * event, gboolean unlock)
     GST_DEBUG_OBJECT (src, "Sending newsegment from %" G_GINT64_FORMAT
         " to %" G_GINT64_FORMAT, src->segment.start, stop);
 
-    gst_pad_push_event (src->srcpad,
-        gst_event_new_new_segment (FALSE,
-            src->segment.rate, src->segment.format,
-            src->segment.last_stop, stop, src->segment.time));
+    if (src->priv->start_segment)
+      gst_event_unref (src->priv->start_segment);
+    src->priv->start_segment =
+        gst_event_new_new_segment_full (FALSE,
+        src->segment.rate, src->segment.applied_rate, src->segment.format,
+        src->segment.last_stop, stop, src->segment.time);
   }
 
   src->priv->discont = TRUE;
@@ -892,9 +912,16 @@ gst_base_src_send_event (GstElement * element, GstEvent * event)
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_FLUSH_START:
     case GST_EVENT_FLUSH_STOP:
+      /* sending random flushes downstream can break stuff,
+       * especially sync since all segment info will get flushed */
       break;
     case GST_EVENT_EOS:
+      /* FIXME, queue EOS and make sure the task or pull function 
+       * perform the EOS actions. */
+      break;
     case GST_EVENT_NEWSEGMENT:
+      /* sending random NEWSEGMENT downstream can break sync. */
+      break;
     case GST_EVENT_TAG:
     case GST_EVENT_BUFFERSIZE:
       break;
@@ -1373,6 +1400,16 @@ gst_base_src_loop (GstPad * pad)
 
   src->priv->last_sent_eos = FALSE;
 
+  /* push events to close/start our segment before we do the real work. */
+  if (src->priv->close_segment) {
+    gst_pad_push_event (pad, src->priv->close_segment);
+    src->priv->close_segment = NULL;
+  }
+  if (src->priv->start_segment) {
+    gst_pad_push_event (pad, src->priv->start_segment);
+    src->priv->start_segment = NULL;
+  }
+
   /* if we operate in bytes, we can calculate an offset */
   if (src->segment.format == GST_FORMAT_BYTES)
     position = src->segment.last_stop;
@@ -1381,16 +1418,11 @@ gst_base_src_loop (GstPad * pad)
 
   ret = gst_base_src_get_range (src, position, src->blocksize, &buf);
   if (G_UNLIKELY (ret != GST_FLOW_OK)) {
-    if (ret == GST_FLOW_UNEXPECTED)
-      goto eos;
-    else {
-      GST_INFO_OBJECT (src, "pausing after gst_base_src_get_range() =  %d",
-          ret);
-      goto pause;
-    }
+    GST_INFO_OBJECT (src, "pausing after gst_base_src_get_range() =  %d", ret);
+    goto pause;
   }
   if (G_UNLIKELY (buf == NULL))
-    goto error;
+    goto null_buffer;
 
   /* figure out the new position */
   switch (src->segment.format) {
@@ -1442,47 +1474,47 @@ gst_base_src_loop (GstPad * pad)
     goto pause;
   }
 
-  if (eos)
-    goto eos;
+  if (eos) {
+    GST_INFO_OBJECT (src, "pausing after EOS");
+    ret = GST_FLOW_UNEXPECTED;
+    goto pause;
+  }
 
 done:
   gst_object_unref (src);
   return;
 
   /* special cases */
-eos:
-  {
-    GST_DEBUG_OBJECT (src, "going to EOS, getrange returned UNEXPECTED");
-    /* we finished the segment */
-    src->data.ABI.running = FALSE;
-    gst_pad_pause_task (pad);
-    if (src->segment.flags & GST_SEEK_FLAG_SEGMENT) {
-      gst_element_post_message (GST_ELEMENT (src),
-          gst_message_new_segment_done (GST_OBJECT (src),
-              src->segment.format, src->segment.last_stop));
-    } else {
-      gst_pad_push_event (pad, gst_event_new_eos ());
-      src->priv->last_sent_eos = TRUE;
-    }
-    goto done;
-  }
 pause:
   {
     const gchar *reason = gst_flow_get_name (ret);
 
     GST_DEBUG_OBJECT (src, "pausing task, reason %s", reason);
+    src->data.ABI.running = FALSE;
     gst_pad_pause_task (pad);
     if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
-      /* for fatal errors we post an error message */
-      GST_ELEMENT_ERROR (src, STREAM, FAILED,
-          (_("Internal data flow error.")),
-          ("streaming task paused, reason %s", reason));
-      gst_pad_push_event (pad, gst_event_new_eos ());
-      src->priv->last_sent_eos = TRUE;
+      if (ret == GST_FLOW_UNEXPECTED) {
+        /* perform EOS logic */
+        if (src->segment.flags & GST_SEEK_FLAG_SEGMENT) {
+          gst_element_post_message (GST_ELEMENT (src),
+              gst_message_new_segment_done (GST_OBJECT (src),
+                  src->segment.format, src->segment.last_stop));
+        } else {
+          gst_pad_push_event (pad, gst_event_new_eos ());
+          src->priv->last_sent_eos = TRUE;
+        }
+      } else {
+        /* for fatal errors we post an error message */
+        GST_ELEMENT_ERROR (src, STREAM, FAILED,
+            (_("Internal data flow error.")),
+            ("streaming task paused, reason %s", reason));
+        gst_pad_push_event (pad, gst_event_new_eos ());
+        src->priv->last_sent_eos = TRUE;
+      }
     }
     goto done;
   }
-error:
+null_buffer:
   {
     GST_ELEMENT_ERROR (src, STREAM, FAILED,
         (_("Internal data flow error.")), ("element returned NULL buffer"));
@@ -1894,6 +1926,9 @@ gst_base_src_change_state (GstElement * element, GstStateChange transition)
       if (!gst_base_src_stop (basesrc))
         goto error_stop;
 
+      /* FIXME, deprecate this behaviour, it is very dangerous.
+       * the prefered way of sending EOS downstream is by sending
+       * the EOS event to the element */
       if (!basesrc->priv->last_sent_eos) {
         GST_DEBUG_OBJECT (basesrc, "Sending EOS event");
         gst_pad_push_event (basesrc->srcpad, gst_event_new_eos ());
@@ -1901,6 +1936,10 @@ gst_base_src_change_state (GstElement * element, GstStateChange transition)
       }
       event_p = &basesrc->data.ABI.pending_seek;
       gst_event_replace (event_p, NULL);
+      event_p = &basesrc->priv->close_segment;
+      gst_event_replace (event_p, NULL);
+      event_p = &basesrc->priv->start_segment;
+      gst_event_replace (event_p, NULL);
       break;
     }
     case GST_STATE_CHANGE_READY_TO_NULL:
