@@ -47,13 +47,13 @@
  * </refsect2>
  */
 
-#include <stdlib.h>
-#include <string.h>
-
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
+#include <stdlib.h>
+#include <string.h>
+
 #include "gstfakesrc.h"
 #include <gst/gstmarshal.h>
 
@@ -65,7 +65,7 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
 GST_DEBUG_CATEGORY_STATIC (gst_fake_src_debug);
 #define GST_CAT_DEFAULT gst_fake_src_debug
 
-static GstElementDetails gst_fake_src_details =
+static const GstElementDetails gst_fake_src_details =
 GST_ELEMENT_DETAILS ("Fake Source",
     "Source",
     "Push empty (no data) buffers around",
@@ -127,7 +127,7 @@ static GType
 gst_fake_src_output_get_type (void)
 {
   static GType fakesrc_output_type = 0;
-  static GEnumValue fakesrc_output[] = {
+  static const GEnumValue fakesrc_output[] = {
     {FAKE_SRC_FIRST_LAST_LOOP, "1", "First-Last loop"},
     {FAKE_SRC_LAST_FIRST_LOOP, "2", "Last-First loop"},
     {FAKE_SRC_PING_PONG, "3", "Ping-Pong"},
@@ -152,7 +152,7 @@ static GType
 gst_fake_src_data_get_type (void)
 {
   static GType fakesrc_data_type = 0;
-  static GEnumValue fakesrc_data[] = {
+  static const GEnumValue fakesrc_data[] = {
     {FAKE_SRC_DATA_ALLOCATE, "Allocate data", "allocate"},
     {FAKE_SRC_DATA_SUBBUFFER, "Subbuffer data", "subbuffer"},
     {0, NULL, NULL},
@@ -170,7 +170,7 @@ static GType
 gst_fake_src_sizetype_get_type (void)
 {
   static GType fakesrc_sizetype_type = 0;
-  static GEnumValue fakesrc_sizetype[] = {
+  static const GEnumValue fakesrc_sizetype[] = {
     {FAKE_SRC_SIZETYPE_EMPTY, "Send empty buffers", "empty"},
     {FAKE_SRC_SIZETYPE_FIXED, "Fixed size buffers (sizemax sized)", "fixed"},
     {FAKE_SRC_SIZETYPE_RANDOM,
@@ -190,7 +190,7 @@ static GType
 gst_fake_src_filltype_get_type (void)
 {
   static GType fakesrc_filltype_type = 0;
-  static GEnumValue fakesrc_filltype[] = {
+  static const GEnumValue fakesrc_filltype[] = {
     {FAKE_SRC_FILLTYPE_NOTHING, "Leave data as malloced", "nothing"},
     {FAKE_SRC_FILLTYPE_ZERO, "Fill buffers with zeros", "zero"},
     {FAKE_SRC_FILLTYPE_RANDOM, "Fill buffers with random crap", "random"},
@@ -215,6 +215,7 @@ gst_fake_src_filltype_get_type (void)
 GST_BOILERPLATE_FULL (GstFakeSrc, gst_fake_src, GstBaseSrc, GST_TYPE_BASE_SRC,
     _do_init);
 
+static void gst_fake_src_finalize (GObject * object);
 static void gst_fake_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_fake_src_get_property (GObject * object, guint prop_id,
@@ -250,9 +251,11 @@ gst_fake_src_class_init (GstFakeSrcClass * klass)
   GstElementClass *gstelement_class;
   GstBaseSrcClass *gstbase_src_class;
 
-  gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
-  gstbase_src_class = (GstBaseSrcClass *) klass;
+  gobject_class = G_OBJECT_CLASS (klass);
+  gstelement_class = GST_ELEMENT_CLASS (klass);
+  gstbase_src_class = GST_BASE_SRC_CLASS (klass);
+
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_fake_src_finalize);
 
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_fake_src_set_property);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_fake_src_get_property);
@@ -260,66 +263,66 @@ gst_fake_src_class_init (GstFakeSrcClass * klass)
 /*
   FIXME: this is not implemented; would make sense once basesrc and fakesrc
   support multiple pads
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_OUTPUT,
+  g_object_class_install_property (gobject_class, PROP_OUTPUT,
       g_param_spec_enum ("output", "output", "Output method (currently unused)",
           GST_TYPE_FAKE_SRC_OUTPUT, DEFAULT_OUTPUT, G_PARAM_READWRITE));
 */
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_DATA,
+  g_object_class_install_property (gobject_class, PROP_DATA,
       g_param_spec_enum ("data", "data", "Data allocation method",
           GST_TYPE_FAKE_SRC_DATA, DEFAULT_DATA, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SIZETYPE,
       g_param_spec_enum ("sizetype", "sizetype",
           "How to determine buffer sizes", GST_TYPE_FAKE_SRC_SIZETYPE,
           DEFAULT_SIZETYPE, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SIZEMIN,
+  g_object_class_install_property (gobject_class, PROP_SIZEMIN,
       g_param_spec_int ("sizemin", "sizemin", "Minimum buffer size", 0,
           G_MAXINT, DEFAULT_SIZEMIN, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SIZEMAX,
+  g_object_class_install_property (gobject_class, PROP_SIZEMAX,
       g_param_spec_int ("sizemax", "sizemax", "Maximum buffer size", 0,
           G_MAXINT, DEFAULT_SIZEMAX, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PARENTSIZE,
+  g_object_class_install_property (gobject_class, PROP_PARENTSIZE,
       g_param_spec_int ("parentsize", "parentsize",
           "Size of parent buffer for sub-buffered allocation", 0, G_MAXINT,
           DEFAULT_PARENTSIZE, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_FILLTYPE,
+  g_object_class_install_property (gobject_class, PROP_FILLTYPE,
       g_param_spec_enum ("filltype", "filltype",
           "How to fill the buffer, if at all", GST_TYPE_FAKE_SRC_FILLTYPE,
           DEFAULT_FILLTYPE, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_DATARATE,
+  g_object_class_install_property (gobject_class, PROP_DATARATE,
       g_param_spec_int ("datarate", "Datarate",
           "Timestamps buffers with number of bytes per second (0 = none)", 0,
           G_MAXINT, DEFAULT_DATARATE, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SYNC,
+  g_object_class_install_property (gobject_class, PROP_SYNC,
       g_param_spec_boolean ("sync", "Sync", "Sync to the clock to the datarate",
           DEFAULT_SYNC, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PATTERN,
+  g_object_class_install_property (gobject_class, PROP_PATTERN,
       g_param_spec_string ("pattern", "pattern", "pattern", DEFAULT_PATTERN,
           G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_LAST_MESSAGE,
+  g_object_class_install_property (gobject_class, PROP_LAST_MESSAGE,
       g_param_spec_string ("last-message", "last-message",
           "The last status message", NULL, G_PARAM_READABLE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SILENT,
+  g_object_class_install_property (gobject_class, PROP_SILENT,
       g_param_spec_boolean ("silent", "Silent",
           "Don't produce last_message events", DEFAULT_SILENT,
           G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SIGNAL_HANDOFFS,
+  g_object_class_install_property (gobject_class, PROP_SIGNAL_HANDOFFS,
       g_param_spec_boolean ("signal-handoffs", "Signal handoffs",
           "Send a signal before pushing the buffer", DEFAULT_SIGNAL_HANDOFFS,
           G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_DUMP,
+  g_object_class_install_property (gobject_class, PROP_DUMP,
       g_param_spec_boolean ("dump", "Dump", "Dump produced bytes to stdout",
           DEFAULT_DUMP, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass),
+  g_object_class_install_property (gobject_class,
       PROP_CAN_ACTIVATE_PUSH,
       g_param_spec_boolean ("can-activate-push", "Can activate push",
           "Can activate in push mode", DEFAULT_CAN_ACTIVATE_PUSH,
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-  g_object_class_install_property (G_OBJECT_CLASS (klass),
+  g_object_class_install_property (gobject_class,
       PROP_CAN_ACTIVATE_PULL,
       g_param_spec_boolean ("can-activate-pull", "Can activate pull",
           "Can activate in pull mode", DEFAULT_CAN_ACTIVATE_PULL,
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_IS_LIVE,
+  g_object_class_install_property (gobject_class, PROP_IS_LIVE,
       g_param_spec_boolean ("is-live", "Is this a live source",
           "True if the element cannot produce data in PAUSED", FALSE,
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
@@ -367,6 +370,22 @@ gst_fake_src_init (GstFakeSrc * fakesrc, GstFakeSrcClass * g_class)
   fakesrc->sync = DEFAULT_SYNC;
 }
 
+static void
+gst_fake_src_finalize (GObject * object)
+{
+  GstFakeSrc *src;
+
+  src = GST_FAKE_SRC (object);
+
+  g_free (src->last_message);
+  if (src->parent) {
+    gst_buffer_unref (src->parent);
+    src->parent = NULL;
+  }
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 static gboolean
 gst_fake_src_event_handler (GstBaseSrc * basesrc, GstEvent * event)
 {
