@@ -31,7 +31,6 @@
 #endif
 
 #include <string.h>
-#include <stdlib.h>
 #include <signal.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -40,9 +39,7 @@
 #include <sys/wait.h>
 #endif
 #include <locale.h>             /* for LC_ALL */
-#include "gst/gst-i18n-app.h"
-
-#include <gst/gst.h>
+#include "tools.h"
 
 /* FIXME: This is just a temporary hack.  We should have a better
  * check for siginfo handling. */
@@ -248,6 +245,21 @@ print_tag (const GstTagList * list, const gchar * tag, gpointer unused)
     if (gst_tag_get_type (tag) == G_TYPE_STRING) {
       if (!gst_tag_list_get_string_index (list, tag, i, &str))
         g_assert_not_reached ();
+    } else if (gst_tag_get_type (tag) == GST_TYPE_BUFFER) {
+      GstBuffer *img;
+
+      img = gst_value_get_buffer (gst_tag_list_get_value_index (list, tag, i));
+      if (img) {
+        gchar *caps_str;
+
+        caps_str = GST_BUFFER_CAPS (img) ?
+            gst_caps_to_string (GST_BUFFER_CAPS (img)) : g_strdup ("unknown");
+        str = g_strdup_printf ("buffer of %u bytes, type: %s",
+            GST_BUFFER_SIZE (img), caps_str);
+        g_free (caps_str);
+      } else {
+        str = g_strdup ("NULL buffer");
+      }
     } else {
       str =
           g_strdup_value_contents (gst_tag_list_get_value_index (list, tag, i));
@@ -512,6 +524,7 @@ main (int argc, char *argv[])
         N_("Do not install a fault handler"), NULL},
     {"trace", 'T', 0, G_OPTION_ARG_NONE, &trace,
         N_("Print alloc trace (if enabled at compile time)"), NULL},
+    GST_TOOLS_GOPTION_VERSION,
     {NULL}
   };
   GOptionContext *ctx;
@@ -530,7 +543,7 @@ main (int argc, char *argv[])
 
   gst_alloc_trace_set_flags_all (GST_ALLOC_TRACE_LIVE);
 
-  ctx = g_option_context_new ("gst-launch");
+  ctx = g_option_context_new ("PIPELINE-DESCRIPTION");
   g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
   g_option_context_add_group (ctx, gst_init_get_option_group ());
   if (!g_option_context_parse (ctx, &argc, &argv, &err)) {
@@ -539,6 +552,8 @@ main (int argc, char *argv[])
   }
   g_option_context_free (ctx);
 
+  gst_tools_print_version ("gst-launch");
+
   /* FIXpopt: strip short args, too. We do it ourselves for now */
   j = 1;
   for (i = 1; i < argc; i++) {
@@ -639,7 +654,7 @@ main (int argc, char *argv[])
         event_loop (pipeline, FALSE, GST_STATE_VOID_PENDING);
         goto end;
       case GST_STATE_CHANGE_NO_PREROLL:
-        fprintf (stderr, _("ERROR: Pipeline can't PREROLL ...\n"));
+        fprintf (stderr, _("Pipeline is live and does not need PREROLL ...\n"));
         break;
       case GST_STATE_CHANGE_ASYNC:
         fprintf (stderr, _("Pipeline is PREROLLING ...\n"));
